/* vim: se fo=tcroq : */
/* Copyright (C) 2016 Penguin Development
 *
 * This file is part of Ising, a D-dimensional Ising model simulator.
 *
 * License GPLv3+: GNU GPL version 3 or later
 * 	<http://gnu.org/licenses/gpl.html>.
 * This is free software: you are free to change and redistribute it.
 * There is NO WARRANTY, to the extent permitted by law.
 *
 * Written by Peter Sterk.
 */
#ifndef UTILITY_H
#define UTILITY_H

#include <type_traits>

/** Utility method to calculate b to the power of e (for positive integers b
 * and e) at compile time.
 * @param b base
 * @param e exponent
 * @return b**e
 */
constexpr long long int cstpow (long long int b, long long int e)
{
    return e == 0 ? 1 : b * cstpow (b, e - 1);
}

/** Utility method to calculate x % y using the "always positive" convention.
 * @param x dividend
 * @param y divisor
 * @param x % y if this is 0 or greater, else y + x % y.
 */
template<typename T> constexpr typename
std::enable_if<std::is_integral<T>::value, T>::type absmod (T x, T y)
{
    return ((x % y) >= 0) ? (x % y) : (y + (x % y));
}

#endif
